/*
 * Copyright (C) 2004  Heinz Mauelshagen, Red Hat GmbH. All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

#ifndef _DMRAID_H_
#define _DMRAID_H_

#include <stdint.h>
#include <sys/types.h>
/* FIXME: avoid more library internals. */
#include "misc/lib_context.h"
#include "display/display.h"
#include "format/format.h"
#include "log/log.h"

/*
 * Library init/exit
 */
int libdmraid_init(struct lib_context **lc, int argc, char **argv);
void libdmraid_exit(struct lib_context *lc);

/*
 * Retrieve version identifiers.
 */
int dm_version(struct lib_context *lc, char *version, size_t size);
const char *libdmraid_date(struct lib_context *lc);
const char *libdmraid_version(struct lib_context *lc);

/*
 * Dealing with formats.
 */
int check_valid_format(struct lib_context *lc, char *fmt);

/*
 * Dealing with devices.
 */
unsigned int count_devices(struct lib_context *lc, enum dev_type type);
int discover_devices(struct lib_context *lc);
void discover_raid_devices(struct lib_context *lc, char **fmt_names,
			   char **devices);
void discover_partitions(struct lib_context *lc);
void free_dev_info(struct lib_context *lc, void *di);

/*
 * Erase ondisk metadata.
 */
int erase_metadata(struct lib_context *lc);

/*
 * Dealing with RAID sets.
 */
const char *get_set_type(struct lib_context *lc, void *rs);
const char *get_set_name(struct lib_context *lc, void *rs);
int group_set(struct lib_context *lc, char *name);

enum set_type {
	SETS,
	PARTITIONS,
};
void process_sets(struct lib_context *lc,
		  int (*func)(struct lib_context *lc, void *rs, int arg),
		  int arg, enum set_type type);
int activate_set(struct lib_context *lc, void *rs);
int deactivate_set(struct lib_context *lc, void *rs);

/*
 * Memory allocation
 */
#ifdef	DEBUG_MALLOC

struct lib_context;
void *_dbg_malloc(size_t size, struct lib_context *lc,
		  const char *who, unsigned int line);
void *_dbg_realloc(void *ptr, size_t size, struct lib_context *lc,
		   const char *who, unsigned int line);
void *_dbg_strdup(void *ptr, struct lib_context *lc,
		  const char *who, unsigned int line);
void _dbg_free(void *ptr, struct lib_context *lc,
	       const char *who, unsigned int line);

#define	dbg_malloc(size)	_dbg_malloc((size), lc, __func__, __LINE__)
#define	dbg_realloc(ptr, size)	_dbg_realloc((ptr), (size), lc, \
					     __func__, __LINE__)
#define	dbg_strdup(ptr)		_dbg_strdup((ptr), lc, __func__, __LINE__)
#define	dbg_free(ptr)		_dbg_free((ptr), lc, __func__, __LINE__)

#else

void *_dbg_malloc(size_t size);
void *_dbg_realloc(void *ptr, size_t size);
void *_dbg_strdup(void *ptr);
void _dbg_free(void *ptr);

#define	dbg_malloc	_dbg_malloc
#define	dbg_realloc	_dbg_realloc
#define	dbg_strdup	_dbg_strdup
#define	dbg_free	_dbg_free

#endif /* #ifdef DEBUG_MALLOC */

#endif
